#include "collection.h"


Collection::Collection( int size, int _delta )
{
	if( size < 0 )
		size = 0;
	maxDataSize = size;

	delta = _delta;
	dataSize = 0;

	if( size > 0 )
		dataPtr = (long*) new void*[maxDataSize];
}

int Collection::Add( void *ptr )
{
	if( dataSize >= maxDataSize )
		if( !AddMemory())
			return 0;

	dataPtr[dataSize++] = (long)ptr;
	
	return dataSize;
}

int Collection::AddMemory()
{
	maxDataSize += delta;

	long *newPtr = (long*) new void*[maxDataSize];
	for(int i=0; i<dataSize; i++ )
		newPtr[i] = dataPtr[i];
	delete dataPtr;

	dataPtr = (long*)new char[sizeof( void* )*maxDataSize];
	for(i=0; i<dataSize; i++ )
		dataPtr[i] = newPtr[i];
	delete newPtr;

	return 1;
}

int Collection::Insert( int i, void * ptr )
{
	CheckBound( i );
	if( i < dataSize )
	{
		dataPtr[i] = (long)ptr;
		return 1;
	}

	return 0;
}

int Collection::Remove( void *ptr )
{
	for( int i=0; i< dataSize; i++ )
	{
		if( ptr == (void*)dataPtr[i] )
		{
			delete (void*) dataPtr[i];
			return ReAllocateData( i );
		}
	}

	return 0;
}

int Collection::Remove( int i )
{
	CheckBound( i );
	delete (void*) dataPtr[i];
	return ReAllocateData( i );
}


void Collection::ChangePlace( int i, int j )
{
	CheckBound( i );
	CheckBound( j );
	
	long l1;
	l1 = dataPtr[i];
	dataPtr[i] = dataPtr[j];
	dataPtr[j] = l1;
}


int Collection::ReAllocateData( int i )
{
	dataSize--;
	for( int ii=i; ii<dataSize; ii++ )
		dataPtr[ii] = dataPtr[ii+1];

	return 1;
}

int Collection::RemoveAll()
{
	dataSize = maxDataSize = 0;
	for(int i=0; i< dataSize; i++ )
		delete (void*) dataPtr[i];
	delete dataPtr;

	return 1;
}

int Collection::Delete( void *ptr )
{
	for( int i=0; i< dataSize; i++ )
	{
		if( ptr == (void*)dataPtr[i] )
			return ReAllocateData( i );
	}

	return 0;
}

int Collection::Delete( int i )
{
	CheckBound( i );
	return ReAllocateData( i );
}

int Collection::DeleteAll()
{
	dataSize = maxDataSize = 0;
	delete dataPtr;

	return 1;
}


void *Collection::operator[](int i)
{
	CheckBound( i );
	return (void*) dataPtr[i];
}

void Collection::operator+=( void *ptr )
{
	Add( ptr );
}


//int operator==(void *, void *);
