#include <stdio.h>
#include <stdlib.h>
#include <dirent.h>
#include <string.h>
#include <unistd.h>
#include <ctype.h>

#include "sortdiffcoll.h"

const char nodelist_mask[] = "[Nn][Oo][Dd][Ee][Ll][Ii][Ss][Tt].???";
const char nodediff_mask[] = "[Nn][Oo][Dd][Ee][Dd][Ii][Ff][Ff].???";
const char nodediffz_mask[] = "[Nn][Oo][Dd][Ee][Dd][Ii][Ff][Ff].[Zz]??";

char *nodelist  = NULL;
char node_path[_MAX_PATH+3] = ".";
char diff_path[_MAX_PATH+3] = ".";

short last_nodelist = -1;
SortedDiffCollection diffcol( 10, 5 );
DIR *dirp = NULL;

short CheckLastNodelist();
void  CheckNodediffs();
void  CheckZippedNodediffs();
int   HandleDiff( int diffindex );
int   CheckOpt( int argc, char **argv );

void Usage()
{
//	printf( " nodediff [-n nodelist_path] [-d nodelist_path] \n");
}

int main( int argc, char **argv )
{
	printf( " nodediff v1.0qnx coded by ccpro \n" );
	
	if( CheckOpt( argc, argv ))
	{
		puts( "\n wrong key " );
		Usage();
		exit( 3 );
	}
	
	dirp = opendir( "." );
	last_nodelist = CheckLastNodelist();
	
	printf( " last nodelist.%03d \n", last_nodelist );

	CheckZippedNodediffs();
	CheckNodediffs();
	printf( "found %d nodediffs \n", diffcol.GetSize()); 
		
	for( int i=0; i<diffcol.GetSize(); i++ )
	{
		if( diffcol.GetDiffNum(i) <= last_nodelist )
		{
			printf( "skipped\n" );
		}else
		{
			if( HandleDiff( i ))
			{
				printf( "error\n" );
				exit( 1 );
			}else
				printf( "ok\n" );
		}
	}

	closedir( dirp );
	return 0;
}


int CheckOpt( int argc, char **argv )
{
	for( int i=1; i< argc; i++ )
	{
		if( !argv[i][0] == '-' )
			return -1;
			
		switch( tolower(argv[i][1]) )
		{
			case 'n':
				
			default:
				return -1;
		}
	}
	
	return 0;
}

static short GetExtensionNum( char *name )
{
	char full_path[ _MAX_PATH ];
	char node[ _MAX_NODE ];
	char dir[ _MAX_DIR ];
	char fname[ _MAX_FNAME ];
	char ext[ _MAX_EXT ];
                    
	_splitpath( name, node, dir, fname, ext );
	
	return (short) atoi( &ext[1] );
}

short CheckLastNodelist()
{
	if( !dirp )
		return 0;

	rewinddir( dirp );

	dirent *direntp;
	short num = -1;
	while( direntp = readdir( dirp ))
	{
		if( fnmatch( nodelist_mask, direntp->d_name, 0 ))
			continue;
			
		short n = GetExtensionNum( direntp->d_name );
		if( num < n )
		{
			num = n;
			if( nodelist )
				delete nodelist;
				
			nodelist = strdup( direntp->d_name );
		}
			
	}
	return num;
	
}

void CheckNodediffs()
{
	if( !dirp )
		return;
		
	rewinddir( dirp );

	dirent *direntp;
	while( direntp = readdir( dirp ))
	{
		if( !fnmatch( nodediff_mask, direntp->d_name, 0 ))
			diffcol += strdup( direntp->d_name );
	}
}


int UnPackZDiff( char *zdiff )
{
	char tmp[1024];
	
	sprintf(tmp, "unzip -o -j -qq %s ", zdiff ); 
	if( system(tmp) != 0 ) 
	{
  		perror( "unpacker returned error" );
		return -1;
	}
	unlink( zdiff );
	
	return 0;
}

void CheckZippedNodediffs()
{
	if( !dirp )
		return;
		
	rewinddir( dirp );

	dirent *direntp;
	while( direntp = readdir( dirp ))
	{
		if( !fnmatch( nodediffz_mask, direntp->d_name, 0 ))
		{
			if( UnPackZDiff( direntp->d_name ))
				exit( 2 );
			
			printf( " %s - processed\n", direntp->d_name );
		}
	}
}

int   HandleDiff( int diffindex )
{
	FILE *oldn = fopen( nodelist, "rt" );
	if( oldn == NULL )
	{
		perror( "open file" );
		return -1;
	}
	
	char *new_nodelist = new char[_MAX_PATH + _MAX_FNAME + 5 ];
	sprintf( new_nodelist, "nodelist.%03d", diffcol.GetDiffNum( diffindex ));
	
	FILE *newn = fopen( new_nodelist, "wt" );
	if( newn == NULL )
	{
		perror( "open file" );
		return -1;
	}

	FILE *diff = fopen( diffcol[diffindex], "rt" );
	if( diff == NULL )
	{
		perror( "open file" );
		return -1;
	}
	
	//
	// handle cicle
	//
	
	// take first string from nodediff & compare 
	// it with first string from nodelist
	
	char strd[1024], strn[1024];
	
	fgets( strd, sizeof( strd ), oldn );
	fgets( strn, sizeof( strn ), diff );
	
	if( strcmp( strd, strn ))
	{
		printf( "nodelist first string \n - %s\n", strn );
		printf( "nodediff first string \n - %s\n", strd );
		return -2;
	}
	
//	fputs( strn, newn );
	int num_str = 0;
	while( fgets( strd, sizeof( strd ), diff ))
	{
		switch( toupper(*strd) )
		{
			case 'C':
			{
				num_str = atoi(&strd[1]);
				for( int i=0; i<num_str; i++ )
				{
					fgets( strn, sizeof( strn ), oldn );
					fputs( strn, newn );
				}
				break;	
			}
			case 'D':
			{
				num_str = atoi(&strd[1]);
				for( int i=0; i<num_str; i++ )
				{
					fgets( strn, sizeof( strn ), oldn );
				}
				break;
			}
			case 'A':
			{
				num_str = atoi(&strd[1]);
				for( int i=0; i<num_str; i++ )
				{
					fgets( strd, sizeof( strd ), diff );
					fputs( strd, newn );
				}
				break;	
			}
		}
	}
	
	fclose( oldn );
	unlink( nodelist );
	fclose( newn );
	fclose( diff );
	unlink( diffcol[diffindex] );
	
	delete nodelist;
	nodelist = new_nodelist;

	return 0;
}
